#ifndef __VSCAN_KAVP_H_
#define __VSCAN_KAVP_H_

#include <includes.h>


#ifdef HAVE_UTIME_H
#include <utime.h>
#endif

#ifdef HAVE_DIRENT_H
#include <dirent.h>
#endif

#ifdef HAVE_FCNTL_H
#include <fcntl.h>
#endif

/* Configuration Section :-) */

/* which samba version is this VFS module compiled for:
 * Set SAMBA_VERSION_MAJOR to 3 for Samba 3.0 or
 * to 2 for Samba 2.2.x
 * Set SAMBA_VERSION_MINOR to 4 for Samba >= 2.2.4 
 * Set it to 2 if you're using Samba 2.2.2/2.2.3
 * Set it to 1 if you're using Samba 2.2.1[a] or 0 for Samba 2.2.0[a] 
 * If SAMBA_VERSION_MAJOR is set to 3, SAMBA_VERSION_MINOR
 * is ignored!
 *
 * Per default, Samba >= 2.2.4 is assumed!
*/

#ifndef SAMBA_VERSION_MAJOR
# define SAMBA_VERSION_MAJOR 2 
#endif

#ifndef SAMBA_VERSION_MINOR
# define SAMBA_VERSION_MINOR 4
#endif 

/* default location of samba-style configuration file (needs Samba >= 2.2.4
 or Samba 3.0 */

#define PARAMCONF "/etc/vscan-kavp.conf"


/* False = log only infected file, True = log every file access */

#ifndef VSCAN_VERBOSE_FILE_LOGGING
# define VSCAN_VERBOSE_FILE_LOGGING False   
#endif

/* if a file is bigger than VSCAN_SOPHOS_MAX_SIZE it won't be scanned. Has to be
   specified in bytes! If it set to 0, the file size check is disabled */

#ifndef VSCAN_MAX_SIZE 
# define VSCAN_MAX_SIZE 0 
#endif

/* True = scan files on open */

#ifndef VSCAN_SCAN_ON_OPEN 
# define VSCAN_SCAN_ON_OPEN True 
#endif

/* True = scan files on close */

#ifndef VSCAN_SCAN_ON_CLOSE
# define VSCAN_SCAN_ON_CLOSE False
#endif


/* True = deny access in case of virus scanning failure */

#ifndef VSCAN_DENY_ACCESS_ON_ERROR
# define VSCAN_DENY_ACCESS_ON_ERROR True
#endif


/* Daemon socket file */

#define AVPCTL "/var/run/AvpCtl"

/* End Configuration Section */


#ifndef SYSLOG_FACILITY
#define SYSLOG_FACILITY   LOG_USER
#endif

#ifndef SYSLOG_PRIORITY
#define SYSLOG_PRIORITY   LOG_NOTICE
#endif

/* virus messages will be logged as SYSLOG_PRIORITY_ALERT */
#ifndef SYSLOG_PRIORITY_ALERT
#define SYSLOG_PRIORITY_ALERT	LOG_ERR
#endif

#ifndef bool 
 #ifndef BOOL_DEFINED
  typedef int bool;
 #endif
#endif 

#ifndef FALSE
# define FALSE  0
#endif /* ! FALSE */
#ifndef TRUE
# define TRUE   1
#endif /* ! TRUE */

/* Globals */
int 	kavp_socket;			/* Remember our socket number  -1 is not socket */
int  vscan_kavp_scanfile(char *scan_file, char* client_ip);
void vscan_kavp_init(void);
void vscan_kavp_end(void);
void vscan_syslog(char *printMessage, ...);
void vscan_syslog_alert(char *printMessage, ...);

#endif /* __VSCAN_KAVP_H */

