/*
 * $Id: vscan-trend.c,v 1.23 2002/11/11 11:37:02 reniar Exp $
 * 
 * virusscanning VFS module for samba.  Log infected files via syslog
 * facility and block access using Trophie.
 *
 * Copyright (C) Rainer Link, 2001-2002 
 *               OpenAntiVirus.org <rainer@openantivirus.org>
 *
 * 
 * based on the audit VFS module by
 * Copyright (C) Tim Potter, 1999-2000
 * Copyright (C) Alexander Bokovoy, 2002
 *
 * based on a Trophie by
 * Copyright (C) Vanja Hrustic, 2001
 *
 * Credits to
 * - Dave Collier-Brown for his VFS tutorial (http://www.geocities.com/orville_torpid/papers/vfs_tutorial.html)
 * - REYNAUD Jean-Samuel for helping me to solve some general Samba VFS issues at the first place
 * - Simon Harrison for his solution without Samba VFS (http://www.smh.uklinux.net/linux/trend.html)
 * - the whole Samba Team :)
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *  
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *  
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */


#include <includes.h>
#include <vfs.h>

#include "vscan-trend.h"


fstring config_file;            /* location of config file, either
                                   PARAMCONF or as set via vfs options
                                */

ssize_t max_size;          	/* do not scan files greater than max_size
                                   if max_size = 0, scan any file
                                */

BOOL verbose_file_logging;  	/* log ever file access */

BOOL scan_on_open;         	/* scan a file before it is opened
                                   Defaults to True
                                */

BOOL scan_on_close;        	/* scan a new file put on share or
                                   if file was modified
                                   Defaults to False
                                */

BOOL deny_access_on_error;	/* if using SAVI fails, should access to any
				   file be denied? */

fstring	trophie_socket_name;	/* name of Trophie socket */


/* module version */
static const char module_id[]="vscan-trend 0.3.0";


#if SAMBA_VERSION_MAJOR!=3
 /* Samba 2.2.x */
 /* Function prototypes */

 #if SAMBA_VERSION_MINOR==4
  /* Samba 2.2.4 */
  int vscan_connect(struct connection_struct *conn, const char *svc, const char *user);
  void vscan_disconnect(struct connection_struct *conn);
  int vscan_open(struct connection_struct *conn, const char *fname, int flags, mode_t mode);
  int vscan_close(struct files_struct *fsp, int fd);
 #else
  /* Samba 2.2.3-2.2.0 */
  int vscan_connect(struct connection_struct *conn, char *svc, char *user);
  void vscan_disconnect(struct connection_struct *conn);
  int vscan_open(struct connection_struct *conn, char *fname, int flags, mode_t mode);
  int vscan_close(struct files_struct *fsp, int fd);
 #endif
#else
 /* Samba 3.0 alphaX */
 static int vscan_connect(struct connection_struct *conn, const char *svc, const char *user); 
 static void vscan_disconnect(struct connection_struct *conn);
 static int vscan_open(struct connection_struct *conn, const char *fname, 
 int flags, mode_t mode);
 static int vscan_close(struct files_struct *fsp, int fd);
#endif
/* VFS operations */

#if SAMBA_VERSION_MAJOR!=3
 /* Samba 2.2.x */
 extern struct vfs_ops default_vfs_ops;   /* For passthrough operation */
#else
 /* Samba 3.0 alphaX */
 static struct vfs_ops default_vfs_ops;   /* For passthrough operation */
 static struct smb_vfs_handle_struct *vscan_handle; /* use skel_handle->data
 for storing per-instance private data */
#endif

#if SAMBA_VERSION_MAJOR!=3 
 /* Samba 2.2.x */
 #if SAMBA_VERSION_MINOR==4
 /* Samba 2.2.4 */
 struct vfs_ops vscan_ops = {
    
	/* Disk operations */

	vscan_connect,			/* connect */
	vscan_disconnect,		/* disconnect */
	NULL,				/* disk free */

	/* Directory operations */

	NULL,				/* opendir */
	NULL,				/* readdir */
	NULL,				/* mkdir */
	NULL,				/* rmdir */
	NULL,				/* closedir */

	/* File operations */

	vscan_open,			/* open */
	vscan_close,			/* close */
	NULL,				/* read  */
	NULL,				/* write */
	NULL,				/* lseek */
	NULL,				/* rename */
	NULL,				/* fsync */
	NULL,				/* stat  */
	NULL,				/* fstat */
	NULL,				/* lstat */
	NULL,				/* unlink */
	NULL,				/* chmod */
	NULL,				/* fchmod */
	NULL,				/* chown */
	NULL,				/* fchown */
	NULL,				/* chdir */
	NULL,				/* getwd */
	NULL,				/* utime */
	NULL,				/* ftruncate */
	NULL,				/* lock */
	NULL,				/* symlink */
	NULL,				/* readlink */
	NULL,				/* link */
	NULL,				/* mknod */
	NULL,				/* realpath */
	NULL,				/* fget_nt_acl */
	NULL,				/* get_nt_acl */
	NULL,				/* fset_nt_acl */
	NULL,				/* set_nt_acl */

	NULL,				/* chmod_acl */
	NULL,				/* fchmod_acl */

	NULL,				/* sys_acl_get_entry */
	NULL,				/* sys_acl_get_tag_type */
	NULL,				/* sys_acl_get_permset */
	NULL,				/* sys_acl_get_qualifier */
	NULL,				/* sys_acl_get_file */
	NULL,				/* sys_acl_get_fd */
	NULL,				/* sys_acl_clear_perms */
	NULL,				/* sys_acl_add_perm */
	NULL,				/* sys_acl_to_text */
	NULL,				/* sys_acl_init */
	NULL,				/* sys_acl_create_entry */
	NULL,				/* sys_acl_set_tag_type */
	NULL,				/* sys_acl_set_qualifier */
	NULL,				/* sys_acl_set_permset */
	NULL,				/* sys_acl_valid */
	NULL,				/* sys_acl_set_file */
	NULL,				/* sys_acl_set_fd */
	NULL,				/* sys_acl_delete_def_file */
	NULL,				/* sys_acl_get_perm */
	NULL,				/* sys_acl_free_text */
	NULL,				/* sys_acl_free_acl */
	NULL				/* sys_acl_free_qualifier */
 };
 #else
 /* Samba 2.2.3-2.2.0 */
 struct vfs_ops vscan_ops = {
    
	/* Disk operations */

	vscan_connect,		  /* connect */
	vscan_disconnect,	  /* disconnect */
	NULL,                     /* disk free */

	/* Directory operations */

	NULL,			  /* opendir */
	NULL,                     /* readdir */
	NULL,			  /* mkdir */
	NULL,			  /* rmdir */	
	NULL,                     /* closedir */

	/* File operations */

	vscan_open,		  /* open  */
	vscan_close,		  /* close */
	NULL,                     /* read  */
	NULL,                     /* write */
	NULL,                     /* lseek */
	NULL,			  /* rname */
	NULL,                     /* fsync */
	NULL,                     /* stat  */
	NULL,                     /* fstat */
	NULL,                     /* lstat */
	NULL,			  /* unlink */
	NULL,			  /* chmod */
	NULL,                     /* chown */
	NULL,                     /* chdir */
	NULL,                     /* getwd */
	NULL,                     /* utime */
	NULL,                     /* ftruncate */
	NULL,                     /* lock */
	NULL,                     /* fget_nt_acl */
	NULL,                     /* get_nt_acl */
	NULL,                     /* fset_nt_acl */
	NULL                      /* set_nt_acl */
 };
 #endif
#else
 /* Samba 3.0 alphaX */
 static vfs_op_tuple vscan_ops[] = {

	/* Disk operations */
	{vscan_connect,		SMB_VFS_OP_CONNECT,	SMB_VFS_LAYER_TRANSPARENT},
	{vscan_disconnect,	SMB_VFS_OP_DISCONNECT,	SMB_VFS_LAYER_TRANSPARENT},

	/* File operations */
	{vscan_open,		SMB_VFS_OP_OPEN,	SMB_VFS_LAYER_TRANSPARENT},
	{vscan_close,		SMB_VFS_OP_CLOSE,	SMB_VFS_LAYER_TRANSPARENT},

	/* Finish VFS operations definition */
	{NULL, 			SMB_VFS_OP_NOOP,	SMB_VFS_LAYER_NOOP}
 };
#endif 


static BOOL set_boolean(BOOL *b, char *value)
{

	BOOL retval;

	retval = True;

	if ( StrCaseCmp("yes", value) == 0 ||
	     StrCaseCmp("true", value) == 0 ||
	     StrCaseCmp("1", value) == 0 )
		*b = True;
	else if ( StrCaseCmp("no", value) == 0 ||
		  StrCaseCmp("false", value) == 0 ||
		  StrCaseCmp("0", value) == 0 )
		*b = False;
	else {
		DEBUG(2, ("samba-vscan: badly formed boolean in configuration file, parameter %s\n", value));
		retval = False;
	}

	return retval; 
}

static BOOL do_parameter(char *param, char *value)
{
	if ( StrCaseCmp("max file size", param) == 0 ) {
		max_size = atoi(value);
		DEBUG(3, ("max file size is: %d\n", max_size));
	} else if ( StrCaseCmp("verbose file logging", param) == 0 ) {
		set_boolean(&verbose_file_logging, value);
		DEBUG(3, ("verbose file logging is: %d\n", verbose_file_logging));	
	} else if ( StrCaseCmp("scan on open", param) == 0 ) {
		set_boolean(&scan_on_open, value);
		DEBUG(3, ("scan on open: %d\n", scan_on_open));
	} else if ( StrCaseCmp("scan on close", param) == 0 ) {
		set_boolean(&scan_on_close, value);
		DEBUG(3, ("scan on open is: %d\n", scan_on_close));
	} else if ( StrCaseCmp("deny access on error", param) == 0 ) {
		set_boolean(&deny_access_on_error, value);
		DEBUG(3, ("deny access on error is: %d\n", deny_access_on_error));
	} else if ( StrCaseCmp("trophie socket name", param) == 0) {
		fstrcpy(trophie_socket_name, value);
		DEBUG(3, ("trophie socket name is %s\n", trophie_socket_name));
	} else
		DEBUG(3, ("unknown parameter: %s", param));

	return True;
}

static BOOL do_section(char *section)
{
	/* simply return true, there's only one section :-) */
	return True;
}


/* Implementation of vfs_ops.  */

#if SAMBA_VERSION_MAJOR==3
 /* Samba 3.0 alphaX */
 static int vscan_connect(struct connection_struct *conn, const char *svc, const char *user)
#else
 /* Samba 2.2.x */
 #if SAMBA_VERSION_MINOR==4
  /* Samba 2.2.4 */
  int vscan_connect(struct connection_struct *conn, const char *svc, const char *user)
 #else
  /* Samba 2.2.3-2.2.0 */
  int vscan_connect(struct connection_struct *conn, char *svc, char *user)
 #endif
#endif
{

	#if (SAMBA_VERSION_MAJOR==2 && SAMBA_VERSION_MINOR==4) || SAMBA_VERSION_MAJOR==3
	  pstring opts_str;
	  char *p;
	#endif

	int retval;

	/* set default value for configuration files */
	fstrcpy(config_file, PARAMCONF);

	/* set default value for max file size */
	max_size = VSCAN_MAX_SIZE;

	/* set default value for file logging */
	verbose_file_logging = VSCAN_VERBOSE_FILE_LOGGING;

	/* set default value for scan on open() */
	scan_on_open = VSCAN_SCAN_ON_OPEN;

	/* set default value for scan on close() */
	scan_on_close = VSCAN_SCAN_ON_CLOSE;

	/* set default value for deny access on error */
	deny_access_on_error = VSCAN_DENY_ACCESS_ON_ERROR;

	/* name of Trophie socket */
	fstrcpy(trophie_socket_name, TROPHIE_SOCKET_NAME);

	vscan_syslog("connect to service %s by user %s\n", 
	       svc, user);

	vscan_trend_init();


	#if (SAMBA_VERSION_MAJOR==2 && SAMBA_VERSION_MINOR==4) || SAMBA_VERSION_MAJOR==3
	  pstrcpy(opts_str, (const char*) lp_vfs_options(SNUM(conn)));
	  if( !*opts_str ) {
		DEBUG(3, ("vscan-trend: no configuration file set - using default value (%s).\n", lp_vfs_options(SNUM(conn))));
	  } else {
		p = opts_str;
		if ( next_token(&p, config_file, "=", sizeof(config_file)) ) {
			trim_string(config_file, " ", " ");
			if ( !strequal("config-file", config_file) ) {
				DEBUG(3, ("samba-vscan - connect: options %s is not config-file\n", config_file));
				/* setting default value */
				fstrcpy(config_file, PARAMCONF);

			} else {
				if ( !next_token(&p, config_file," \n",sizeof(config_file)) ) {
					DEBUG(3, ("samba-vscan - connect: no option after config-file=\n"));
				/* setting default value */
				fstrcpy(config_file, PARAMCONF);
				} else {
					trim_string(config_file, " ", " ");
					DEBUG(3, ("samba-vscan - connect: config file name is %s\n", config_file));
				}
			}
		}
	  }

  	  retval = pm_process(config_file, do_section, do_parameter);
	  DEBUG(10, ("pm_process returned %d\n", retval));			
	  if (!retval) vscan_syslog("ERROR: could not parse configuration file '%s'. File not found or not read-able. Using compiled-in defaults", config_file);
	#endif
				
	#if SAMBA_VERSION_MAJOR!=3
	return default_vfs_ops.connect(conn, svc, user);
	#else
	return 0;	/* Success */
	#endif
}

#if SAMBA_VERSION_MAJOR==3
 /* Samba 3.0 alphaX */
 static void vscan_disconnect(struct connection_struct *conn)
#else
 /* Samba 2.2.x */
 void vscan_disconnect(struct connection_struct *conn)
#endif
{
	vscan_syslog("disconnected\n");
	default_vfs_ops.disconnect(conn);
}


#if SAMBA_VERSION_MAJOR==3
 /* Samba 3.0 alphaX */
 static int vscan_open(struct connection_struct *conn, const char *fname, int flags, mode_t mode)
#else
 /* Samba 2.2.x */
 #if SAMBA_VERSION_MINOR==4
  /* Samba 2.2.4 */
  int vscan_open(struct connection_struct *conn, const char *fname, int flags, mode_t mode)
 #else
  /* Samba 2.2.3 - 2.2.0 */
  int vscan_open(struct connection_struct *conn, char *fname, int flags, mode_t mode)
 #endif
#endif
{

	int retval;
	SMB_STRUCT_STAT stat_buf;
	char client_ip[18];
	int sockfd;
	pstring filepath;

	/* scan files while opening? */
	if ( !scan_on_open ) {
		DEBUG(3, ("samba-vscan - open: File '%s' not scanned as scan_on_open is not set\n", fname));
		return default_vfs_ops.open(conn, fname, flags, mode);
	}

        if ( (default_vfs_ops.stat(conn, fname, &stat_buf)) != 0 ) {   
		/* an error occured */
		DEBUG(5, ("samba-vscan - open: stat failed. Error is %s\n", strerror(errno))); 
		return default_vfs_ops.open(conn, fname, flags, mode);

	} else if ( S_ISDIR(stat_buf.st_mode) ) {
		/* is it a directory? */
		DEBUG(5, ("samba-vscan - open: refused to scan directory\n"));
		 return default_vfs_ops.open(conn, fname, flags, mode);
	} else if ( stat_buf.st_size == 0 ) {
		/* do not scan empty files */
		DEBUG(5, ("samba-vscan - open: refused to scan empty file %s\n", fname));
		 return default_vfs_ops.open(conn, fname, flags, mode);
	} else if ( ( stat_buf.st_size > max_size ) && ( max_size > 0 ) ) /* file is too large */
		vscan_syslog("File %s too large! Not scanned!", fname);
	else if ( stat_buf.st_size == 0 ) /* do not scan empty files */
		return default_vfs_ops.open(conn, fname, flags, mode);
	else {
               /* open socket */
                sockfd = vscan_trend_init();

                if ( sockfd == -1 && deny_access_on_error ) {
                        /* an error occured - can not communicate to daemon - de
ny access */
                        vscan_syslog("ERROR: can not communicate to Trophie - access to file %s denied", fname);
			 errno = EACCES;
                        return -1;
                } else if ( sockfd >= 0 ) {
			               
                        /* Assemble complete file path */       
                        fstrcpy(filepath, conn->connectpath);
                        fstrcat(filepath, "/"); 
                        fstrcat(filepath, fname);       

			fstrcpy(client_ip, conn->client_address);
			retval = vscan_trend_scanfile(sockfd, filepath, client_ip);
			if ( retval == -1 && deny_access_on_error ) {
			/* an error occured - can not communicate to dae mon - deny access */
                                vscan_syslog("ERROR: can not communicate to Soph
ie - access to file %s denied", fname);
                                errno = EACCES;
                                return -1;

			} else if ( retval == 1 ) {
				/* virus found */
				vscan_trend_end(sockfd);
				errno = EACCES; 
				return -1;
			}
		}

		/* close socket */
		vscan_trend_end(sockfd);
	}

	return default_vfs_ops.open(conn, fname, flags, mode);

}


#if SAMBA_VERSION_MAJOR==3
 /* Samba 3.0 alphaX */
 static int vscan_close(struct files_struct *fsp, int fd)
#else
 /* Samba 2.2.x */
 int vscan_close(struct files_struct *fsp, int fd)
#endif
{
        pstring filepath;
        int retval, rv;
	char client_ip[18];
	int sockfd;

        /* First close the file */
        retval = default_vfs_ops.close(fsp, fd);

	if ( !scan_on_close ) {
		DEBUG(3, ("samba-vscan - close: File '%s' not scanned as scan_on_close is not set\n", fsp->fsp_name));
		return retval;
	}
 

	/* get the file name */
        fstrcpy(filepath, fsp->conn->connectpath);
        fstrcat(filepath, "/"); 
        fstrcat(filepath, fsp->fsp_name);        
	
        /* Don't scan directorys */
        if ( fsp->is_directory )
            return retval;


	if ( !fsp->modified ) {
		if ( verbose_file_logging )
			vscan_syslog("file %s was not modified - not scanned", filepath);

		return retval;
	}

	sockfd = vscan_trend_init();
        if ( sockfd >= 0 ) {
		fstrcpy(client_ip, fsp->conn->client_address);
		/* scan only file, do nothing */
                rv = vscan_trend_scanfile(sockfd, filepath, client_ip);
                vscan_trend_end(sockfd);
        }       

	return retval;

}

/* VFS initialisation function.  Return initialised vfs_ops structure
   back to SAMBA. */

#if SAMBA_VERSION_MAJOR==3
 /* Samba 3.0 alphaX */
 vfs_op_tuple *vfs_init(int *vfs_version, struct vfs_ops *def_vfs_ops,
			struct smb_vfs_handle_struct *vfs_handle)
#else
 /* Samba 2.2.x */
 #if SAMBA_VERSION_MINOR==4   
  /* Samba 2.2.4 */
  struct vfs_ops *vfs_init(int *vfs_version, struct vfs_ops *def_vfs_ops)
 #elif SAMBA_VERSION_MINOR==2
  /* Samba 2.2.2 / Samba 2.2.3 !!! */
  struct vfs_ops *vfs_init(int* Version, struct vfs_ops *ops)
 #elif SAMBA_VERSION_MINOR==1
  /* Samba 2.2.1 */
  struct vfs_ops *vfs_module_init(int *vfs_version)
 #else
  /* Samba 2.2.0 */
  struct vfs_ops *vfs_init(int *vfs_version)
 #endif
#endif
{

	#if SAMBA_VERSION_MAJOR!=3
	 #if SAMBA_VERSION_MINOR==4
	  /* Samba 2.2.4 */
	  struct vfs_ops tmp_ops;
	 #endif
	#endif
 
        openlog("smbd_vscan_trend", LOG_PID, SYSLOG_FACILITY);

        #if SAMBA_VERSION_MAJOR==3
         /* Samba 3.0 alphaX */
         *vfs_version = SMB_VFS_INTERFACE_VERSION;
         vscan_syslog("samba-vscan (%s) loaded (Samba 3.x), (c) by Rainer Link, OpenAntiVirus.org", module_id);
        #else
         /* Samba 2.2.x */
         #if SAMBA_VERSION_MINOR==4
          /* Samba 2.2.4 */
          *vfs_version = SMB_VFS_INTERFACE_VERSION;
          vscan_syslog("samba-vscan (%s) loaded (Samba >=2.2.4), (c) by Rainer Link, OpenAntiVirus.org", module_id);
         #elif SAMBA_VERSION_MINOR==2
          /* Samba 2.2.2 / Samba 2.2.3 !!! */
          *Version = SMB_VFS_INTERFACE_VERSION;
          vscan_syslog("samba-vscan (%s) loaded (Samba 2.2.2/2.2.3), (c) by Rainer Link, OpenAntiVirus.org", module_id);
         #else
          /* Samba 2.2.1 / Samba 2.2.0 */
          *vfs_version = SMB_VFS_INTERFACE_VERSION;
          vscan_syslog("samba-vscan (%s) loaded (Samba 2.2.0/2.2.1), (c) by Rainer Link, OpenAntiVirus.org",
               module_id);
         #endif
        #endif


	#if SAMBA_VERSION_MAJOR==3
         /* Samba 3.0 alphaX */
	 DEBUG(3, ("Initialising default vfs hooks\n"));
         memcpy(&default_vfs_ops, def_vfs_ops, sizeof(struct vfs_ops));

         /* Remember vfs_handle for further allocation and referencing of 
	    private information in vfs_handle->data
         */
	 vscan_handle = vfs_handle;
	 return vscan_ops;
        #else
         /* Samba 2.2.x */
	 #if SAMBA_VERSION_MINOR==4
	  /* Samba 2.2.4 */

	  *vfs_version = SMB_VFS_INTERFACE_VERSION;
	  memcpy(&tmp_ops, def_vfs_ops, sizeof(struct vfs_ops));
	  tmp_ops.connect = vscan_connect;
	  tmp_ops.disconnect = vscan_disconnect;
	  tmp_ops.open = vscan_open;
	  tmp_ops.close = vscan_close;
	  memcpy(&vscan_ops, &tmp_ops, sizeof(struct vfs_ops));
	  return(&vscan_ops);

	 #else
          /* Samba 2.2.3-2.2.0 */
          return(&vscan_ops);
	 #endif
	#endif
}

#if SAMBA_VERSION_MAJOR==3
/* VFS finalization function */
void vfs_done(connection_struct *conn)
{
        DEBUG(3, ("Finalizing default vfs hooks\n"));
}
#endif

