/* 
 * Core Interface for SAVI			
 *
 * Copyright (C) Rainer Link, 2001-2002
 *               OpenAntiVirus.org <rainer@openantivirus.org>
 *
 * This stuff is heavily based on Sophie by
 * Copyright (C) Vanja Hrustic, 2001
 *
 * This software is licensed under the GNU General Public License (GPL)
 * See: http://www.gnu.org/copyleft/gpl.html
 *
*/

#include "vscan-sophos_core.h"
 

CISavi2 *pSAVI = NULL;

bool savi_ok = TRUE;

/* Added by Kevin Spicer 9 July 2002
 * This counts the number of connect / disconnect messages */
int VscanConnects = 0;

/* print a message via syslog */
void vscan_syslog(char *printMessage, ...)
{
        char printMsg[512];

        va_list argptr;
        va_start(argptr, printMessage);
        vsnprintf(printMsg, sizeof(printMsg)-1, printMessage, argptr);
        va_end(argptr);

	syslog(SYSLOG_PRIORITY, "%s", printMsg);
}

/* print a message via syslog */
void vscan_syslog_alert(char *printMessage, ...)
{
        char printMsg[512];

        va_list argptr;
        va_start(argptr, printMessage);
        vsnprintf(printMsg, sizeof(printMsg)-1, printMessage, argptr);
        va_end(argptr);

	syslog(SYSLOG_PRIORITY_ALERT, "%s", printMsg);
}



/* initialise SAVI */
void vscan_sophos_init(void)
{
        const char *ClientName = "vscan-sophos";

        CISweepClassFactory2 *pFactory;
        HRESULT hr;

	/* Check if this is a second or further call to this function which
	 * should be ignored
	 * Added by Kevin Spicer 9 July 2002 */
	if ( VscanConnects > 0 ) 
	{
                vscan_syslog("INFORMATION: Additional connect request ignored\n");
		VscanConnects++;
		return;
	} else {
		VscanConnects = 1;
	}
	/* End of addition */

        hr = DllGetClassObject((REFIID)&SOPHOS_CLSID_SAVI2, (REFIID)&SOPHOS_IID_CLASSFACTORY2, (void **) &pFactory);

        if (!(hr == SOPHOS_S_OK))
        {
                vscan_syslog("ERROR: Could not initialize SAVI class/object\n");
	        savi_ok = FALSE; 
        }
        else
        {
                hr = pFactory->pVtbl->CreateInstance(pFactory, NULL, &SOPHOS_IID_SAVI2, (void **) &pSAVI );
                        
                pFactory->pVtbl->Release(pFactory);
        
                if (hr == SOPHOS_S_OK)
                {
                        hr = pSAVI->pVtbl->InitialiseWithMoniker(pSAVI, ClientName);
                
                        if (SOPHOS_FAILED(hr))
                        {
                                vscan_syslog("ERROR: Failed to initialize SAVI [%ld]", (long) hr);
                                pSAVI->pVtbl->Release(pSAVI);
                                pSAVI= NULL;
                                savi_ok = FALSE;
                        } else {
                        	/* Engine configuration */
                        	hr = pSAVI->pVtbl->SetConfigValue(pSAVI, SOPHOS_DO_FULL_SWEEP, SOPHOS_TYPE_U32, VSCAN_SOPHOS_DO_FULL_SWEEP);
                        	if (SOPHOS_FAILED(hr))
                                	vscan_syslog("NOTICE: Unable to setup configuration value SOPHOS_DO_FULL_SWEEP to %s",
                                        VSCAN_SOPHOS_DO_FULL_SWEEP);
                        
                        	hr = pSAVI->pVtbl->SetConfigValue(pSAVI, SOPHOS_DYNAMIC_DECOMPRESSION, SOPHOS_TYPE_U32, VSCAN_SOPHOS_DYNAMIC_DECOMPRESSION);
                        	if (SOPHOS_FAILED(hr))
                                	vscan_syslog("NOTICE: Unable to setup configuration value SOPHOS_DYNAMIC_DECOMPRESSION to %s",
                                        VSCAN_SOPHOS_DYNAMIC_DECOMPRESSION);
                        
                        	hr = pSAVI->pVtbl->SetConfigValue(pSAVI, SOPHOS_FULL_MACRO_SWEEP, SOPHOS_TYPE_U32, VSCAN_SOPHOS_FULL_MACRO_SWEEP);
                        	if (SOPHOS_FAILED(hr))
                                	vscan_syslog("NOTICE: Unable to setup configuration value SOPHOS_FULL_MACRO_SWEEP to %s",
                                        VSCAN_SOPHOS_FULL_MACRO_SWEEP);
                        
                        	hr = pSAVI->pVtbl->SetConfigValue(pSAVI, SOPHOS_IGNORE_TEMPLATE_BIT, SOPHOS_TYPE_U32, VSCAN_SOPHOS_IGNORE_TEMPLATE_BIT);
                        	if (SOPHOS_FAILED(hr))
                                	vscan_syslog("NOTICE: Unable to setup configuration value SOPHOS_IGNORE_TEMPLATE_BIT to %s",
                                        VSCAN_SOPHOS_IGNORE_TEMPLATE_BIT);
                        
                        	hr = pSAVI->pVtbl->SetConfigValue(pSAVI, SOPHOS_OLE2_HANDLING, SOPHOS_TYPE_U32, VSCAN_SOPHOS_OLE2_HANDLING);
                        	if (SOPHOS_FAILED(hr))
                                	vscan_syslog("NOTICE: Unable to setup configuration value SOPHOS_OLE2_HANDLING to %s",
                                        VSCAN_SOPHOS_OLE2_HANDLING);
                        
                        	hr = pSAVI->pVtbl->SetConfigValue(pSAVI, SOPHOS_VBA3_HANDLING, SOPHOS_TYPE_U32, VSCAN_SOPHOS_VBA3_HANDLING);
                        	if (SOPHOS_FAILED(hr))
                                	vscan_syslog("NOTICE: Unable to setup configuration value SOPHOS_VBA3_HANDLING to %s",
                                        VSCAN_SOPHOS_VBA3_HANDLING);
                        
                        	hr = pSAVI->pVtbl->SetConfigValue(pSAVI, SOPHOS_VBA5_HANDLING, SOPHOS_TYPE_U32, VSCAN_SOPHOS_VBA5_HANDLING);
                        	if (SOPHOS_FAILED(hr))
                                	vscan_syslog("NOTICE: Unable to setup configuration value SOPHOS_VBA5_HANDLING to %s",
                                        VSCAN_SOPHOS_VBA5_HANDLING);
                        
                        	hr = pSAVI->pVtbl->SetConfigValue(pSAVI, SOPHOS_DECOMPRESS_VBA5, SOPHOS_TYPE_U32, VSCAN_SOPHOS_DECOMPRESS_VBA5);
                        	if (SOPHOS_FAILED(hr))
                                	vscan_syslog("NOTICE: Unable to setup configuration value SOPHOS_DECOMPRESS_VBA5 to %s",
                                        VSCAN_SOPHOS_DECOMPRESS_VBA5);
                        	hr = pSAVI->pVtbl->SetConfigValue(pSAVI, SOPHOS_DO_EMULATION, SOPHOS_TYPE_U32, VSCAN_SOPHOS_DO_EMULATION);
                        	if (SOPHOS_FAILED(hr))
                                	vscan_syslog("NOTICE: Unable to setup configuration value SOPHOS_DO_EMULATION to %s",
                                        VSCAN_SOPHOS_DO_EMULATION);
                        
                        	hr = pSAVI->pVtbl->SetConfigValue(pSAVI, SOPHOS_PE_HANDLING, SOPHOS_TYPE_U32, VSCAN_SOPHOS_PE_HANDLING);
                        	if (SOPHOS_FAILED(hr))
                                	vscan_syslog("NOTICE: Unable to setup configuration value SOPHOS_PE_HANDLING to %s",
                                        VSCAN_SOPHOS_PE_HANDLING);
                        
                        	hr = pSAVI->pVtbl->SetConfigValue(pSAVI, SOPHOS_XF_HANDLING, SOPHOS_TYPE_U32, VSCAN_SOPHOS_XF_HANDLING);
                        	if (SOPHOS_FAILED(hr))
                                	vscan_syslog("NOTICE: Unable to setup configuration value SOPHOS_XF_HANDLING to %s",
                                        VSCAN_SOPHOS_XF_HANDLING);
                        
                        	hr = pSAVI->pVtbl->SetConfigValue(pSAVI, SOPHOS_PM97_HANDLING, SOPHOS_TYPE_U32, VSCAN_SOPHOS_PM97_HANDLING);
                        	if (SOPHOS_FAILED(hr))
                                	vscan_syslog("NOTICE: Unable to setup configuration value SOPHOS_PM97_HANDLING to %s",
                                        VSCAN_SOPHOS_PM97_HANDLING);
                        
                        	hr = pSAVI->pVtbl->SetConfigValue(pSAVI, SOPHOS_ZIP_DECOMPRESSION, SOPHOS_TYPE_U32, VSCAN_SOPHOS_ZIP_DECOMPRESSION);
                        	if (SOPHOS_FAILED(hr))
                                	vscan_syslog("NOTICE: Unable to setup configuration value SOPHOS_ZIP_DECOMPRESSION to %s",
                                        VSCAN_SOPHOS_ZIP_DECOMPRESSION);
                        
                        	hr = pSAVI->pVtbl->SetConfigValue(pSAVI, SOPHOS_ARJ_DECOMPRESSION, SOPHOS_TYPE_U32, VSCAN_SOPHOS_ARJ_DECOMPRESSION);
                        	if (SOPHOS_FAILED(hr))
                                	vscan_syslog("NOTICE: Unable to setup configuration value SOPHOS_ARJ_DECOMPRESSION to %s",
                                        VSCAN_SOPHOS_ARJ_DECOMPRESSION);
                        
                        	hr = pSAVI->pVtbl->SetConfigValue(pSAVI, SOPHOS_RAR_DECOMPRESSION, SOPHOS_TYPE_U32, VSCAN_SOPHOS_RAR_DECOMPRESSION);
                        	if (SOPHOS_FAILED(hr))
                                	vscan_syslog("NOTICE: Unable to setup configuration value SOPHOS_RAR_DECOMPRESSION to %s",
                                        VSCAN_SOPHOS_RAR_DECOMPRESSION);
                        
                        	hr = pSAVI->pVtbl->SetConfigValue(pSAVI, SOPHOS_UUE_DECOMPRESSION, SOPHOS_TYPE_U32, VSCAN_SOPHOS_UUE_DECOMPRESSION);
                        	if (SOPHOS_FAILED(hr))
                                	vscan_syslog("NOTICE: Unable to setup configuration value SOPHOS_UUE_DECOMPRESSION to %s",
                                        VSCAN_SOPHOS_UUE_DECOMPRESSION);
                        
                        	hr = pSAVI->pVtbl->SetConfigValue(pSAVI, SOPHOS_GZIP_DECOMPRESSION, SOPHOS_TYPE_U32, VSCAN_SOPHOS_GZIP_DECOMPRESSION);
                        	if (SOPHOS_FAILED(hr))
                                	vscan_syslog("NOTICE: Unable to setup configuration value SOPHOS_GZIP_DECOMPRESSION to %s",
                                        VSCAN_SOPHOS_GZIP_DECOMPRESSION);
                        
                        	hr = pSAVI->pVtbl->SetConfigValue(pSAVI, SOPHOS_TAR_DECOMPRESSION, SOPHOS_TYPE_U32, VSCAN_SOPHOS_TAR_DECOMPRESSION);
                        	if (SOPHOS_FAILED(hr))
                                	vscan_syslog("NOTICE: Unable to setup configuration value SOPHOS_TAR_DECOMPRESSION to %s",
                                        VSCAN_SOPHOS_TAR_DECOMPRESSION);
                        
                        	hr = pSAVI->pVtbl->SetConfigValue(pSAVI, SOPHOS_CMZ_DECOMPRESSION, SOPHOS_TYPE_U32, VSCAN_SOPHOS_CMZ_DECOMPRESSION);
                        	if (SOPHOS_FAILED(hr))
                                	vscan_syslog("NOTICE: Unable to setup configuration value SOPHOS_CMZ_DECOMPRESSION to %s",
                                        VSCAN_SOPHOS_CMZ_DECOMPRESSION);
                        
                        	hr = pSAVI->pVtbl->SetConfigValue(pSAVI, SOPHOS_MAX_RECURSION_DEPTH, SOPHOS_TYPE_U16, VSCAN_SOPHOS_MAX_RECURSION_DEPTH);
                        	if (SOPHOS_FAILED(hr))
                                	vscan_syslog("NOTICE: Unable to setup configuration value SOPHOS_MAX_RECURSION_DEPTH to %s",
                                        VSCAN_SOPHOS_MAX_RECURSION_DEPTH);

                        	/*
                         	 These are the ones not found in SAVI include files - I added them since they were listed in configuration output
                        	*/
                        
                        	hr = pSAVI->pVtbl->SetConfigValue(pSAVI, SOPHOS_LHA_DECOMPRESSION, SOPHOS_TYPE_U32, VSCAN_SOPHOS_LHA_DECOMPRESSION);
                        	if (SOPHOS_FAILED(hr))
                                	vscan_syslog("NOTICE: Unable to setup configuration value SOPHOS_LHA_DECOMPRESSION to %s",
                                        VSCAN_SOPHOS_LHA_DECOMPRESSION);

                        	hr = pSAVI->pVtbl->SetConfigValue(pSAVI, SOPHOS_SFX_HANDLING, SOPHOS_TYPE_U32, VSCAN_SOPHOS_SFX_HANDLING);
                        	if (SOPHOS_FAILED(hr))
                                	vscan_syslog("NOTICE: Unable to setup configuration value SOPHOS_SFX_HANDLING to %s",
                                        VSCAN_SOPHOS_SFX_HANDLING);

/* seems not to be supported under UNIX
                        hr = pSAVI->pVtbl->SetConfigValue(pSAVI, SOPHOS_MSCABINET_HANDLING, SOPHOS_TYPE_U32, VSCAN_SOPHOS_MSCABINET_HANDLING);
                        if (SOPHOS_FAILED(hr))
                                vscan_syslog(0, "NOTICE: Unable to setup configuration value SOPHOS_MSCABINET_HANDLING to %s",
                                        VSCAN_SOPHOS_MSCABINET_HANDLING);
*/
                        	hr = pSAVI->pVtbl->SetConfigValue(pSAVI, SOPHOS_TNEF_HANDLING, SOPHOS_TYPE_U32, VSCAN_SOPHOS_TNEF_HANDLING);
                        	if (SOPHOS_FAILED(hr))
                                	vscan_syslog("NOTICE: Unable to setup configuration value SOPHOS_TNEF_HANDLING to %s",
                                        VSCAN_SOPHOS_TNEF_HANDLING);

                       	 	hr = pSAVI->pVtbl->SetConfigValue(pSAVI, SOPHOS_MSCOMPRESS_HANDLING, SOPHOS_TYPE_U32, VSCAN_SOPHOS_MSCOMPRESS_HANDLING);
                       	if (SOPHOS_FAILED(hr))
                                vscan_syslog("NOTICE: Unable to setup configuration value SOPHOS_MSCOMPRESS_HANDLING to %s",
                                        VSCAN_SOPHOS_MSCOMPRESS_HANDLING);

                        	hr = pSAVI->pVtbl->SetConfigValue(pSAVI, SOPHOS_OF95DECRYPT_HANDLING, SOPHOS_TYPE_U32, VSCAN_SOPHOS_OF95DECRYPT_HANDLING);
                        	if (SOPHOS_FAILED(hr))
                                	vscan_syslog(0, "NOTICE: Unable to setup configuration value SOPHOS_OF95DECRYPT_HANDLING to %s",
                                        VSCAN_SOPHOS_OF95DECRYPT_HANDLING);

                        	hr = pSAVI->pVtbl->SetConfigValue(pSAVI, SOPHOS_DELETE_ALL_MACROS, SOPHOS_TYPE_U32, VSCAN_SOPHOS_DELETE_ALL_MACROS);
                        	if (SOPHOS_FAILED(hr))
                                	vscan_syslog("NOTICE: Unable to setup configuration value SOPHOS_DELETE_ALL_MACROS to %s",
                                        VSCAN_SOPHOS_DELETE_ALL_MACROS);

                        /* 29-Jul-2001 - for IDE 3.48, Engine 2.5 */
                        	hr = pSAVI->pVtbl->SetConfigValue(pSAVI, SOPHOS_VBE, SOPHOS_TYPE_U32, VSCAN_SOPHOS_VBE);
                        	if (SOPHOS_FAILED(hr))
                                	vscan_syslog("NOTICE: Unable to setup configuration value SOPHOS_VBE to %s",
                                        VSCAN_SOPHOS_VBE);

                        	hr = pSAVI->pVtbl->SetConfigValue(pSAVI, SOPHOS_EXEC_FILE_DISINFECTION, SOPHOS_TYPE_U32, VSCAN_SOPHOS_EXEC_FILE_DISINFECT);
                        	if (SOPHOS_FAILED(hr))
                                	vscan_syslog("NOTICE: Unable to setup configuration value SOPHOS_EXEC_FILE_DISINFECTION to %s",
                                        VSCAN_SOPHOS_EXEC_FILE_DISINFECT);

                        	hr = pSAVI->pVtbl->SetConfigValue(pSAVI, SOPHOS_VISIO_FILE_HANDLING, SOPHOS_TYPE_U32, VSCAN_SOPHOS_VISIO_FILE_HANDLING);
                        	if (SOPHOS_FAILED(hr))
                                	vscan_syslog("NOTICE: Unable to setup configuration value SOPHOS_VISIO_FILE_HANDLING to %s",
                                        VSCAN_SOPHOS_VISIO_FILE_HANDLING);
			}
                }

        }
}

/*
void vscan_sophos_syslog(char *syslogMessage, ...)
{
        char logMsg[512];

        va_list argptr;
        va_start(argptr, syslogMessage);
        vsnprintf(logMsg, sizeof(logMsg)-1, syslogMessage, argptr);
        va_end(argptr);

        syslog(SYSLOG_PRIORITY, "%s\n", logMsg);
}

*/

/*
  If virus is found, logs the filename/virusname into syslog
*/
/* void vscan_sophos_log_virus(char *infected_file, CIEnumSweepResults *scan_results) CHANGED to the following by Kevin Spicer 23/6/02 */
void vscan_sophos_log_virus(char *infected_file, CIEnumSweepResults *scan_results, char *client_ip)
{
        HRESULT hr;

        OLECHAR virusName[256];
        CISweepResults *details = NULL;
        SOPHOS_ULONG fetched;

        while (scan_results->pVtbl->Next(scan_results, 1, (void **) &details, &fetched) == SOPHOS_S_OK)
        {
                hr = details->pVtbl->GetVirusName(details, sizeof(virusName)-1, (LPOLESTR) virusName, NULL);

                if (SOPHOS_FAILED(hr))
                {
/* Kevin Spicer changed vscan_syslog() to vscan_syslog_alert() 23/6/02 and added client_ip*/
                        vscan_syslog_alert("ERROR: Could not get a virus name for infected file '%s' Client: '%s'", infected_file, client_ip);
                }
                else
                {
/* Kevin Spicer changed vscan_syslog() to vscan_syslog_alert() 23/6/02 and added client_ip*/
                        vscan_syslog_alert("Scan result: '%s' infected with virus '%s' Client '%s'", infected_file, virusName, client_ip);
                }
	/* Change the permissions of the file to none
	 * added by Kevin Spicer 23/6/02 */
	chmod(infected_file, 0); 
        }
        
}



/*
  Scans a file (*FILE*, not a directory - keep that in mind) for a virus
*/
/* int vscan_sophos_scanfile(char *scan_file) CHANGED to the following by Kevin Spicer 23/6/02 */
int vscan_sophos_scanfile(char *scan_file, char *client_ip)
{
        HRESULT hr;
        
        CIEnumSweepResults *scan_results = NULL;

	#if VSCAN_SOPHOS_VERBOSE_FILE_LOGGING==1
        vscan_syslog("Scanning file : '%s'", scan_file);
	#endif

        hr = pSAVI->pVtbl->SweepFile(pSAVI, scan_file, (REFIID)&SOPHOS_IID_ENUM_SWEEPRESULTS, (void **) &scan_results);

        /* Ouch. Making this one did hurt ;) */
        switch (hr)
        {
                case SOPHOS_S_OK:
		#if VSCAN_SOPHOS_VERBOSE_FILE_LOGGING==1
		vscan_syslog("file %s is clean", scan_file);
		#endif
                return 0;
                break;
                
                case SOPHOS_SAVI2_ERROR_VIRUSPRESENT:
                vscan_sophos_log_virus(scan_file, scan_results, client_ip);
                return 1;
                break;
                
                case SOPHOS_SAVI2_ERROR_INITIALISING:
                vscan_syslog("Error: SAVI initialization failed");
                return -1;
                
                case SOPHOS_SAVI2_ERROR_TERMINATING:
                vscan_syslog("Error: Error occured while unloading SAVI");
                return -1;
                
                case SOPHOS_SAVI2_ERROR_SWEEPFAILURE:
                vscan_syslog("Error: Virus scan failed");
                return -1;
                
                case SOPHOS_SAVI2_ERROR_NOT_INITIALISED:
                vscan_syslog("Error: Engine not initialized");
                return -1;

                case SOPHOS_SAVI2_ERROR_IC_INCOMPATIBLE_VERSION:
                vscan_syslog("Error: InterCheck client not compatible with this version of SAV");
                return -1;
                
                case SOPHOS_SAVI2_ERROR_IC_ACCESS_DENIED:
                vscan_syslog("Error: Insufficient right - unable to disable InterCheck client");
                return -1;
                
                case SOPHOS_SAVI2_ERROR_IC_SCAN_PREVENTED:
                vscan_syslog("Error: InterCheck client could not be disabled.  Request to scan the file has been denied.");
                return -1;
                
                case SOPHOS_SAVI2_ERROR_DISINFECTION_FAILED:
                vscan_syslog("Error: Disinfection failed");
                return -1;
                
                case SOPHOS_SAVI2_ERROR_DISINFECTION_UNAVAILABLE:
                vscan_syslog("Error: Disinfection attempted on an uninfected file");
                return -1;
                
                case SOPHOS_SAVI2_ERROR_UPGRADE_FAILED:
                vscan_syslog("Error: Attempt to upgrade virus engine failed");
                return -1;
                
                case SOPHOS_SAVI2_ERROR_SAV_NOT_INSTALLED:
                vscan_syslog("Error: Sophos Anti-Virus is not installed");
                return -1;
                
                case SOPHOS_SAVI2_ERROR_INVALID_CONFIG_NAME:
                vscan_syslog("Error: Attempt to get/set SAVI configuration with incorrect name");
                return -1;
                
                case SOPHOS_SAVI2_ERROR_INVALID_CONFIG_TYPE:
                vscan_syslog("Error: Attempt to get/set SAVI configuration with incorrect type");
                return -1;
                
                case SOPHOS_SAVI2_ERROR_INIT_CONFIGURATION:
                vscan_syslog("Error: Could not configure SAVI");
                return -1;

                case SOPHOS_SAVI2_ERROR_NOT_SUPPORTED:
                vscan_syslog("Error: Not supported in this SAVI implementation");
                return -1;
               
                case SOPHOS_SAVI2_ERROR_COULD_NOT_OPEN:
                vscan_syslog("Error: File could not be accessed");
                return -1;
               
                default:
                return -1;
        }
}

/*
  Cleanup
*/
void vscan_sophos_end(void)
{
	/* Check if there was more than one call to vscan_sophos_init()
	 * If there was all but the last call to this func should be ignored
	 * Added by Kevin Spicer 9 July 2002 */
	if ( VscanConnects > 1 ) 
	{
                vscan_syslog("INFORMATION: Additional disconnect request ignored\n");
		VscanConnects--;
		return;
	} else {
		/* redundant? */
		VscanConnects = 0;
	}
	/* End of addition */

        if (pSAVI)
        {
                pSAVI->pVtbl->Terminate(pSAVI);
                pSAVI->pVtbl->Release(pSAVI);
        }

        vscan_syslog("pSAVI cleaned up and released/terminated");

}
